.\" -*- nroff -*-
.ds g g
.ds G G
.TH \*GINDXBIB @MAN1EXT@ "@MDATE@" "Groff Version @VERSION@"
.SH NAME
\*gindxbib \- make inverted index for bibliographic databases
.SH SYNOPSIS
.ad l
.nr i \n(.i
.in +\w'\fB\*gindxbib 'u
.ti \niu
.B \*gindxbib
.de OP
.ie \\n(.$-1 .RI "[\ \fB\\$1\fP" "\\$2" "\ ]"
.el .RB "[\ " "\\$1" "\ ]"
..
.OP \-vw
.OP \-c file
.OP \-d dir
.OP \-f file
.OP \-h n
.OP \-i string
.OP \-k n
.OP \-l n
.OP \-n n
.OP \-o file
.OP \-t n
.RI [\  filename \|.\|.\|.\ ]
.ad b
.SH DESCRIPTION
.B \*gindxbib
makes an inverted index for the bibliographic databases in
.IR filename \|.\|.\|.
for use with
.BR \*grefer (@MAN1EXT@),
.BR \*glookbib (@MAN1EXT@),
and
.BR \*glkbib (@MAN1EXT@).
The index will be named
.IB filename @INDEX_SUFFIX@\fR;
the index is written to a temporary file which is then renamed to this.
If no filenames are given on the command line because the
.B \-f
option has been used, and no
.B \-o
option is given, the index will be named
.BR @DEFAULT_INDEX_NAME@@INDEX_SUFFIX@ .
.LP
Bibliographic databases are divided into records by blank lines.
Within a record, each fields starts with a
.B %
character at the beginning of a line.
Fields have a one letter name which follows the
.B %
character.
.LP
The values set by the
.BR \-c ,
.BR \-n ,
.BR \-l
and
.B \-t
options are stored in the index;
when the index is searched, keys will be discarded and truncated in a
manner appropriate to these options;
the original keys will be used for verifying that any record
found using the index actually contains the keys.
This means that a user of an index need not know whether these
options were used in the creation of the index,
provided that not all the keys to be searched for
would have been discarded during indexing
and that the user supplies at least the part of each key
that would have remained after being truncated during indexing.
The value set by the
.B \-i
option is also stored in the index
and will be used in verifying records found using the index.
.SH OPTIONS
.TP
.B \-v
Print the version number.
.TP
.B \-w
Index whole files.
Each file is a separate record.
.TP
.BI \-c file
Read the list of common words from
.I file
instead of
.BR @COMMON_WORDS_FILE@ .
.TP
.BI \-d dir
Use
.I dir
as the pathname of the current working directory to store in the index,
instead of the path printed by
.BR pwd (1).
Usually
.I dir
will be a symbolic link that points to the directory printed by
.BR pwd (1).
.TP
.BI \-f file
Read the files to be indexed from
.IR file .
If
.I file
is
.BR \- ,
files will be read from the standard input.
The
.B \-f
option can be given at most once.
.TP
.BI \-i string
Don't index the contents of fields whose names are in
.IR string .
Initially
.I string
is
.B XYZ .
.TP
.BI \-h n
Use the first prime greater than or equal to
.I n
for the size of the hash table.
Larger values of
.I n
will usually make searching faster,
but will make the index larger
and
.B \*gindxbib
use more memory.
Initially
.I n
is 997.
.TP
.BI \-k n
Use at most
.I n
keys per input record.
Initially
.I n
is 100.
.TP
.BI \-l n
Discard keys that are shorter than
.IR n .
Initially
.I n
is 3.
.TP
.BI \-n n
Discard the
.I n
most common words.
Initially
.I n
is 100.
.TP
.BI \-o basename
The index should be named
.IB basename @INDEX_SUFFIX@\fR.
.TP
.BI \-t n
Truncate keys to
.IR n .
Initially
.I n
is 6.
.SH FILES
.TP \w'\fBindxbib\fIXXXXXX'u+2n
.IB filename @INDEX_SUFFIX@
Index.
.TP
.B @DEFAULT_INDEX_NAME@@INDEX_SUFFIX@
Default index name.
.TP
.B @COMMON_WORDS_FILE@
List of common words.
.TP
.BI indxbib XXXXXX
Temporary file.
.SH "SEE ALSO"
.BR \*grefer (@MAN1EXT@),
.BR \*glkbib (@MAN1EXT@),
.BR \*glookbib (@MAN1EXT@)
