/* SeqCltn.c -- implementation of abstract sequential collections

	THIS SOFTWARE FITS THE DESCRIPTION IN THE U.S. COPYRIGHT ACT OF A
	"UNITED STATES GOVERNMENT WORK".  IT WAS WRITTEN AS A PART OF THE
	AUTHOR'S OFFICIAL DUTIES AS A GOVERNMENT EMPLOYEE.  THIS MEANS IT
	CANNOT BE COPYRIGHTED.  THIS SOFTWARE IS FREELY AVAILABLE TO THE
	PUBLIC FOR USE WITHOUT A COPYRIGHT NOTICE, AND THERE ARE NO
	RESTRICTIONS ON ITS USE, NOW OR SUBSEQUENTLY.

Author:
	K. E. Gorlen
	Bg. 12A, Rm. 2033
	Computer Systems Laboratory
	Division of Computer Research and Technology
	National Institutes of Health
	Bethesda, Maryland 20892
	Phone: (301) 496-1111
	uucp: uunet!nih-csl!keith
	Internet: keith@alw.nih.gov
	September, 1985

Function:
	
SeqCltn is an abstract class representing collections whose elements are
ordered and are externally named by integer indices.

$Log:	SeqCltn.c,v $
 * Revision 2.204  89/10/07  23:21:31  keith
 * Pre-release
 * 
 * Revision 2.203.1.1  89/09/09  20:12:58  keith
 * Implement species().
 * Change isEqual() to do assertArgClass() test on argument.
 * 
 * Revision 2.203  89/08/08  15:24:14  keith
 * Pre-release
 * 
 * Revision 2.202.1.3  89/07/07  14:03:50  keith
 * Clean up cast-away const where possible
 * 
 * Revision 2.202.1.2  89/07/03  15:51:10  keith
 * Replace explicit use of Iterator with DO ... OD.
 * 
 * Revision 2.202.1.1  89/07/01  21:55:57  keith
 * Base revision for R2.00 MI version
 * 
 * Revision 2.202  89/06/22  20:55:59  keith
 * Base revision for AT&T C++ R2.0 release (Cycle 20)
 * 
 * Revision 2.201.1.3  89/06/22  10:17:11  keith
 * Remove unnecessary copy constructors.
 * 
 * Revision 2.201.1.2  89/06/21  11:57:00  keith
 * Make all constructors non-inline.
 * Add explicit base class names to constructor initializer lists.
 * 
 * Revision 2.201.1.1  89/05/25  09:57:16  ted
 * Changed casts to call castdown.
 * 
 * Revision 2.201  89/05/12  11:20:18  keith
 * Release for R2.0 Beta test.
 * 
 * Revision 2.200.1.3  89/05/12  11:01:26  keith
 * Revised Object I/O.
 * 
 * Revision 2.200.1.2  89/05/03  23:10:12  keith
 * Utilize abstract classes.
 * 
 * Revision 2.200.1.1  89/04/24  17:18:13  keith
 * Working revision for R2.0 Beta 6++
 * 
 * Revision 2.200  89/04/17  23:31:16  keith
 * Base revision for R2.0 Beta 6.
 * 
 * Revision 2.121  89/02/16  11:09:10  keith
 * Base revision for C++ R1.2.1 compatible version.
 * 
*/

#include "SeqCltn.h"

#define	THIS	SeqCltn
#define	BASE	Collection
#define BASE_CLASSES BASE::desc()
#define MEMBER_CLASSES
#define VIRTUAL_BASE_CLASSES

DEFINE_ABSTRACT_CLASS(SeqCltn,0,"$Header: SeqCltn.c,v 2.204 89/10/07 23:21:31 keith Stab $",NULL,NULL);

extern const int NIHCL_RDABSTCLASS,NIHCL_INDEXRANGE;

SeqCltn::SeqCltn() {}

SeqCltn::SeqCltn(OIOifd& fd) : BASE(fd) {}

SeqCltn::SeqCltn(OIOin& strm) : BASE(strm) {}

int SeqCltn::compare(const Object& arg) const
// Compare two sequenced collections.  If *this > arg return >0,
// *this == arg return 0, and if *this < arg return <0.
{
	assertArgClass(arg,*SeqCltn::desc(),"compare");
	Iterator i(*this);
	Iterator j(castdown(arg));
	Object*	p;	// pointer to next object in this SeqCltn
	Object*	q;	// pointer to next object in arg SeqCltn

	while ((q = j++, p = i++)) {
// previous elements compared equal; longer SeqCltn is therefore larger
		if (q == 0) return 1;
// compare() != 0 at any element determines ordering
		int val;
		if ((val = p->compare(*q)) != 0) return val;
	}
// all elements in this SeqCltn compare() equal to arg SeqCltn
	if (q == 0) return 0;	// size() == arg.size()
	return -1;
}

Object* SeqCltn::first() const { return (Object*)at(0); }
	
unsigned SeqCltn::hash() const
{
	unsigned h = size();
	DO(*this,Object,p) h ^= p->hash(); OD
	return h;
}

int SeqCltn::indexOf(const Object& ob) const
{
	int i = 0;
	DO(*this,Object,p)
		if (p->isEqual(ob)) return i;
		i++;
	OD
	return -1;
}
	
bool SeqCltn::isEqual(const Object& ob) const
{
	assertArgClass(ob,*SeqCltn::desc(),"isEqual");
	if (size() != ob.size()) return NO;
	Iterator i(*this);
	Iterator j(castdown(ob));
	Object* p;
	while (p = i++) if (!p->isEqual(*(j++))) return NO;
	return YES;
}

const Class* SeqCltn::species() const	{ return &classDesc; }

Object* SeqCltn::last() const { return (Object*)at(size()-1); }

unsigned SeqCltn::occurrencesOf(const Object& ob) const
{
	unsigned n = 0;
	DO(*this,Object,p) if (p->isEqual(ob)) n++; OD
	return n;
}
	
Object* SeqCltn::doNext(Iterator& pos) const
{
	if (pos.index < size()) return (Object*)at(pos.index++);
	return 0;
}

void SeqCltn::indexRangeErr() const
{
	setError(NIHCL_INDEXRANGE,DEFAULT,this,className());
}
